package be.iminds.ilabt.jfed.util;

import javax.annotation.Nonnull;
import java.io.IOException;

/**
 * ExceptionUtil: Small util class to detect different types of common exceptions
 */
public class ExceptionUtil {
    private ExceptionUtil() { }

    public static boolean isOutOfMemory(@Nonnull Throwable uncaughtThrowable) {
        //OutOfMemoryError can be nested!

        Throwable c = uncaughtThrowable;
        while (c != null) {
            if (c instanceof OutOfMemoryError) {
                return true;
            }
            c = c.getCause();
        }
        return false;
    }

    public static boolean isOutOfDiskSpace(@Nonnull Throwable uncaughtThrowable) {
        Throwable c = uncaughtThrowable;
        while (c != null) {
            if (c instanceof IOException) {
                //these 3 messages were seen in the wild:

                //windows
                if (c.getMessage().equals("There is not enough space on the disk"))
                    return true;
                //linux
                if (c.getMessage().equals("No space left on device"))
                    return true;
                //posix?
                if (c.getMessage().equals("Not enough space"))
                    return true;
            }
            c = c.getCause();
        }
        return false;
    }
}
